"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.enableHttpGetRpcApi = exports.enableHttpPostRpcApi = void 0;
var rpc_1 = require("../../../common/rpc");
var util_1 = require("../util");
var util_2 = require("./util");
var enableHttpPostRpcApi = function (params) {
    var uws = params.uws, _a = params.route, route = _a === void 0 ? '/rpc/*' : _a, createContext = params.createContext, caller = params.caller;
    uws.post(route, function (res, req) {
        var url = req.getUrl();
        var ctx = createContext(req, res);
        res.onAborted(function () {
            res.aborted = true;
        });
        util_1.readBody(res, function (buffer) {
            processHttpRpcRequest(res, ctx, url, buffer, caller);
        });
    });
};
exports.enableHttpPostRpcApi = enableHttpPostRpcApi;
var enableHttpGetRpcApi = function (params) {
    var uws = params.uws, _a = params.route, route = _a === void 0 ? '/rpc/*' : _a, createContext = params.createContext, caller = params.caller;
    uws.get(route, function (res, req) {
        var url = req.getUrl();
        var query = req.getQuery();
        var params = new URLSearchParams(query);
        var body = String(params.get('a') || 'null');
        var ctx = createContext(req, res);
        res.onAborted(function () {
            res.aborted = true;
        });
        processHttpRpcRequest(res, ctx, url, body, caller);
    });
};
exports.enableHttpGetRpcApi = enableHttpGetRpcApi;
var sendError = function (res, error, pretty) {
    if (res.aborted)
        return;
    res.cork(function () {
        var errorFormatted = rpc_1.formatError(error);
        var body = JSON.stringify(errorFormatted, null, pretty ? 4 : 0);
        res.writeStatus('400 Bad Request').writeHeader('Content-Type', 'application/json').end(body);
    });
};
function processHttpRpcRequest(res, ctx, url, body, caller) {
    try {
        var name_1 = url.substr(5);
        var json = util_2.parsePayload(ctx, body);
        caller.call(name_1, json, ctx)
            .then(function (result) {
            if (res.aborted)
                return;
            res.cork(function () {
                var method = caller.get(name_1);
                var formatted = JSON.stringify(result, null, method.pretty ? 4 : 0);
                res.writeStatus('200 OK').writeHeader('Content-Type', 'application/json').end(formatted);
            });
        })
            .catch(function (error) {
            sendError(res, error, caller.get(name_1).pretty);
        });
    }
    catch (_a) {
        var error = new Error('Could not parse payload');
        sendError(res, error, false);
    }
}
;
